var TSMap = /** @class */ (function () {
    function TSMap(inputMap) {
        var t = this;
        t._keys = [];
        t._values = [];
        t.length = 0;
        if (inputMap) {
            inputMap.forEach(function (v, k) {
                t.set(v[0], v[1]);
            });
        }
    }
    /**
     * Get an array of keys in the map.
     *
     * @returns {Array<K>}
     *
     * @memberOf TSMap
     */
    TSMap.prototype.keys = function () {
        return [].slice.call(this._keys);
    };
    /**
     * Get an array of the values in the map.
     *
     * @returns {Array<V>}
     *
     * @memberOf TSMap
     */
    TSMap.prototype.values = function () {
        return [].slice.call(this._values);
    };
    /**
     * Check to see if an item in the map exists given it's key.
     *
     * @param {K} key
     * @returns {Boolean}
     *
     * @memberOf TSMap
     */
    TSMap.prototype.has = function (key) {
        return this._keys.indexOf(key) > -1;
    };
    /**
     * Get a specific item from the map given it's key.
     *
     * @param {K} key
     * @returns {V}
     *
     * @memberOf TSMap
     */
    TSMap.prototype.get = function (key) {
        if (key == undefined) {
            return undefined;
        }
        var i = this._keys.indexOf(key);
        return i > -1 ? this._values[i] : undefined;
    };
    /**
     * Get a specific key from the map given it's item.
     *
     * @param {V} item
     * @returns {K}
     *
     * @memberOf TSMap
     */
    TSMap.prototype.getKey = function (item) {
        if (item == undefined) {
            return undefined;
        }
        var i = this._values.indexOf(item);
        return i > -1 ? this._keys[i] : undefined;
    };
    /**
     * Set a specific item in the map given it's key, automatically adds new items as needed.
     * Ovewrrites existing items
     *
     * @param {K} key
     * @param {V} value
     *
     * @memberOf TSMap
     */
    TSMap.prototype.set = function (key, value) {
        var t = this;
        // check if key exists and overwrite
        var i = this._keys.indexOf(key);
        if (i > -1) {
            t._values[i] = value;
        }
        else {
            t._keys.push(key);
            t._values.push(value);
            t.length = t._values.length;
        }
        return this;
    };
    /**
     * Provide a number representing the number of items in the map
     *
     * @returns {number}
     *
     * @memberOf TSMap
     */
    TSMap.prototype.size = function () {
        return this.length;
    };
    /**
     * Clear all the contents of the map
     *
     * @returns {TSMap<K,V>}
     *
     * @memberOf TSMap
     */
    TSMap.prototype.clear = function () {
        var t = this;
        t._keys.length = t.length = t._values.length = 0;
        return this;
    };
    /**
     * Delete an item from the map given it's key
     *
     * @param {K} key
     * @returns {Boolean}
     *
     * @memberOf TSMap
     */
    TSMap.prototype["delete"] = function (key) {
        var t = this;
        var i = t._keys.indexOf(key);
        if (i > -1) {
            t._keys.splice(i, 1);
            t._values.splice(i, 1);
            t.length = t._keys.length;
            return true;
        }
        return false;
    };
    return TSMap;
}());
// System LogLevels
//  Off = 0,
//  Low = 1,
//  Medium = 2,
//  High = 3
var MessageType;
(function (MessageType) {
    MessageType[MessageType["Error"] = 1] = "Error";
    MessageType[MessageType["Info"] = 2] = "Info";
    MessageType[MessageType["Trace"] = 3] = "Trace";
})(MessageType || (MessageType = {}));
var Logger = /** @class */ (function () {
    function Logger(prefix, enableTrace) {
        this._enableTrace = enableTrace;
        this._prefix = prefix;
    }
    Logger.prototype.logError = function (message, context) {
        this.logInternal(MessageType.Error, message, context);
    };
    Logger.prototype.logInfo = function (message, context) {
        this.logInternal(MessageType.Info, message, context);
    };
    Logger.prototype.logTrace = function (message, context) {
        this.logInternal(MessageType.Trace, message, context);
    };
    Logger.prototype.logInternal = function (messageType, message, context) {
        if (context === void 0) { context = ''; }
        if (this._enableTrace) {
            var traceMessage = this._prefix + ' [' + MessageType[messageType] + '] ';
            if (context) {
                traceMessage += 'Context: [' + context + ']';
            }
            traceMessage += ' - ' + message;
            System.PrintMultiline(traceMessage);
        }
        if (System.LogLevel >= messageType) {
            System.LogInfo(messageType, message);
        }
    };
    return Logger;
}());
var GlobalHandlerMap = /** @class */ (function () {
    function GlobalHandlerMap() {
        this._handlesToValues = {};
        this.count = 0;
    }
    GlobalHandlerMap.prototype.getMappedValueFromHandle = function (handle) {
        if (handle.toString() in this._handlesToValues) {
            return this._handlesToValues[handle.toString()];
        }
        else {
            return null;
        }
    };
    GlobalHandlerMap.prototype.register = function (handle, mappedValue) {
        this._handlesToValues[handle.toString()] = mappedValue;
        this.count++;
    };
    GlobalHandlerMap.prototype.remove = function (handle) {
        delete this._handlesToValues[handle.toString()];
        this.count--;
    };
    return GlobalHandlerMap;
}());
var CommandClass;
(function (CommandClass) {
    CommandClass[CommandClass["Read"] = 0] = "Read";
    CommandClass[CommandClass["Write"] = 1] = "Write";
})(CommandClass || (CommandClass = {}));
var CommandType;
(function (CommandType) {
    CommandType[CommandType["RRT"] = 0] = "RRT";
    CommandType[CommandType["WRT"] = 1] = "WRT";
    CommandType[CommandType["RAS"] = 2] = "RAS";
    CommandType[CommandType["RIT"] = 3] = "RIT";
    CommandType[CommandType["RMS"] = 4] = "RMS";
    CommandType[CommandType["RFM"] = 5] = "RFM";
    CommandType[CommandType["RRS"] = 6] = "RRS";
    CommandType[CommandType["RSS"] = 7] = "RSS";
    CommandType[CommandType["WMS"] = 8] = "WMS";
    CommandType[CommandType["WFM"] = 9] = "WFM";
    CommandType[CommandType["WOS"] = 10] = "WOS";
    CommandType[CommandType["ROCC"] = 11] = "ROCC";
    CommandType[CommandType["RLIO"] = 12] = "RLIO";
    CommandType[CommandType["RLIT"] = 13] = "RLIT";
    CommandType[CommandType["RLR"] = 14] = "RLR";
    CommandType[CommandType["RLY"] = 15] = "RLY";
    CommandType[CommandType["RLG"] = 16] = "RLG";
    CommandType[CommandType["RNS"] = 17] = "RNS";
    CommandType[CommandType["RNMS"] = 18] = "RNMS";
    CommandType[CommandType["RNFM"] = 19] = "RNFM";
    CommandType[CommandType["RNDS"] = 20] = "RNDS";
    CommandType[CommandType["RNCD"] = 21] = "RNCD";
    CommandType[CommandType["RNHD"] = 22] = "RNHD";
    CommandType[CommandType["RNCN"] = 23] = "RNCN";
    CommandType[CommandType["RNHN"] = 24] = "RNHN";
    CommandType[CommandType["WNS"] = 25] = "WNS";
    CommandType[CommandType["WNMS"] = 26] = "WNMS";
    CommandType[CommandType["WNFM"] = 27] = "WNFM";
    CommandType[CommandType["WNDS"] = 28] = "WNDS";
    CommandType[CommandType["WNCD"] = 29] = "WNCD";
    CommandType[CommandType["WNHD"] = 30] = "WNHD";
    CommandType[CommandType["WNCN"] = 31] = "WNCN";
    CommandType[CommandType["WNHN"] = 32] = "WNHN";
    CommandType[CommandType["WOC"] = 33] = "WOC";
    CommandType[CommandType["WOH"] = 34] = "WOH";
    CommandType[CommandType["RMTG"] = 35] = "RMTG";
    CommandType[CommandType["RMTY"] = 36] = "RMTY";
    CommandType[CommandType["WMTG"] = 37] = "WMTG";
    CommandType[CommandType["WMTY"] = 38] = "WMTY";
    CommandType[CommandType["RMTN"] = 39] = "RMTN";
    CommandType[CommandType["RMTST"] = 40] = "RMTST";
    CommandType[CommandType["RMTZC"] = 41] = "RMTZC";
    CommandType[CommandType["RMTE"] = 42] = "RMTE";
    CommandType[CommandType["WMTN"] = 43] = "WMTN";
    CommandType[CommandType["WMTST"] = 44] = "WMTST";
    CommandType[CommandType["WMTZC"] = 45] = "WMTZC";
    CommandType[CommandType["WMTE"] = 46] = "WMTE";
    CommandType[CommandType["ROP"] = 47] = "ROP";
    CommandType[CommandType["RCH"] = 48] = "RCH";
    CommandType[CommandType["RCL"] = 49] = "RCL";
    CommandType[CommandType["RHH"] = 50] = "RHH";
    CommandType[CommandType["RHL"] = 51] = "RHL";
    CommandType[CommandType["RBH"] = 52] = "RBH";
    CommandType[CommandType["RBL"] = 53] = "RBL";
    CommandType[CommandType["RBS"] = 54] = "RBS";
    CommandType[CommandType["WOP"] = 55] = "WOP";
    CommandType[CommandType["RECO"] = 56] = "RECO";
    CommandType[CommandType["RECL"] = 57] = "RECL";
    CommandType[CommandType["WECO"] = 58] = "WECO";
    CommandType[CommandType["WECL"] = 59] = "WECL";
    CommandType[CommandType["RDC"] = 60] = "RDC";
    CommandType[CommandType["RTO"] = 61] = "RTO";
    CommandType[CommandType["RTS"] = 62] = "RTS";
    CommandType[CommandType["RMLS"] = 63] = "RMLS";
    CommandType[CommandType["WMLS"] = 64] = "WMLS";
    CommandType[CommandType["WML"] = 65] = "WML";
    CommandType[CommandType["RDS"] = 66] = "RDS";
    CommandType[CommandType["REV"] = 67] = "REV";
    CommandType[CommandType["RFV"] = 68] = "RFV";
    CommandType[CommandType["RRIA"] = 69] = "RRIA";
    CommandType[CommandType["RRIS"] = 70] = "RRIS";
    CommandType[CommandType["RRIV"] = 71] = "RRIV";
    CommandType[CommandType["RROS"] = 72] = "RROS";
    CommandType[CommandType["RROV"] = 73] = "RROV";
    CommandType[CommandType["RROE"] = 74] = "RROE";
    CommandType[CommandType["RRHS"] = 75] = "RRHS";
    CommandType[CommandType["RRHSS"] = 76] = "RRHSS";
    CommandType[CommandType["RRHV"] = 77] = "RRHV";
    CommandType[CommandType["RRSS"] = 78] = "RRSS";
    CommandType[CommandType["RRSV"] = 79] = "RRSV";
    CommandType[CommandType["RRRS"] = 80] = "RRRS";
    CommandType[CommandType["RRRV"] = 81] = "RRRV";
    CommandType[CommandType["RRXS"] = 82] = "RRXS";
    CommandType[CommandType["RRXV"] = 83] = "RRXV";
    CommandType[CommandType["RRPS"] = 84] = "RRPS";
    CommandType[CommandType["RRPV"] = 85] = "RRPV";
    CommandType[CommandType["RRWS"] = 86] = "RRWS";
    CommandType[CommandType["RRWV"] = 87] = "RRWV";
    CommandType[CommandType["RRWE"] = 88] = "RRWE";
    CommandType[CommandType["RRA"] = 89] = "RRA";
    CommandType[CommandType["WRHE"] = 90] = "WRHE";
    CommandType[CommandType["RRHN"] = 91] = "RRHN";
    CommandType[CommandType["WRHN"] = 92] = "WRHN";
    CommandType[CommandType["RRHE"] = 93] = "RRHE";
    CommandType[CommandType["RMNS"] = 94] = "RMNS";
    CommandType[CommandType["RMWS"] = 95] = "RMWS";
    CommandType[CommandType["RMRS"] = 96] = "RMRS";
    CommandType[CommandType["RMND"] = 97] = "RMND";
    CommandType[CommandType["RMMI"] = 98] = "RMMI";
    CommandType[CommandType["WMNS"] = 99] = "WMNS";
    CommandType[CommandType["WMWS"] = 100] = "WMWS";
    CommandType[CommandType["WMRS"] = 101] = "WMRS";
    CommandType[CommandType["WMND"] = 102] = "WMND";
    CommandType[CommandType["WMRN"] = 103] = "WMRN";
    CommandType[CommandType["RMESD"] = 104] = "RMESD";
    CommandType[CommandType["RMES"] = 105] = "RMES";
    CommandType[CommandType["RMUS"] = 106] = "RMUS";
    CommandType[CommandType["RMCS"] = 107] = "RMCS";
    CommandType[CommandType["WMESD"] = 108] = "WMESD";
    CommandType[CommandType["WMES"] = 109] = "WMES";
    CommandType[CommandType["RMRF"] = 110] = "RMRF";
    CommandType[CommandType["RMRLS"] = 111] = "RMRLS";
    CommandType[CommandType["RMDS"] = 112] = "RMDS";
    CommandType[CommandType["RMDHS"] = 113] = "RMDHS";
    CommandType[CommandType["RMHS"] = 114] = "RMHS";
    CommandType[CommandType["RMUVC"] = 115] = "RMUVC";
    CommandType[CommandType["RMCR"] = 116] = "RMCR";
    CommandType[CommandType["RMCD"] = 117] = "RMCD";
    CommandType[CommandType["ROU"] = 118] = "ROU";
    CommandType[CommandType["WCH"] = 119] = "WCH";
    CommandType[CommandType["WCL"] = 120] = "WCL";
    CommandType[CommandType["WHH"] = 121] = "WHH";
    CommandType[CommandType["WHL"] = 122] = "WHL";
    CommandType[CommandType["WMRF"] = 123] = "WMRF";
    CommandType[CommandType["WMRLS"] = 124] = "WMRLS";
    CommandType[CommandType["WMDS"] = 125] = "WMDS";
    CommandType[CommandType["WMDHS"] = 126] = "WMDHS";
    CommandType[CommandType["WMHS"] = 127] = "WMHS";
    CommandType[CommandType["WMUVC"] = 128] = "WMUVC";
    CommandType[CommandType["WMCR"] = 129] = "WMCR";
    CommandType[CommandType["WMCD"] = 130] = "WMCD";
    CommandType[CommandType["WOU"] = 131] = "WOU";
    CommandType[CommandType["WTS"] = 132] = "WTS";
    CommandType[CommandType["RMOH"] = 133] = "RMOH";
    CommandType[CommandType["ROR"] = 134] = "ROR";
    CommandType[CommandType["WOR"] = 135] = "WOR"; // Write Override Remaining
})(CommandType || (CommandType = {}));
var ConnectionState;
(function (ConnectionState) {
    ConnectionState[ConnectionState["Failed"] = 0] = "Failed";
    ConnectionState[ConnectionState["Disconnected"] = 1] = "Disconnected";
    ConnectionState[ConnectionState["Connected"] = 2] = "Connected";
})(ConnectionState || (ConnectionState = {}));
var CountryValues = ['CA', 'MX', 'US'];
var DayNigthValues = ['DAY', 'NIGHT'];
var EnabledDisabledValues = ['ENABLED', 'DISABLED'];
var FanModeValues = ['AUTO', 'ON'];
var FanStatusValues = ['FAN AUTO', 'FAN ON', 'FAN RECIRC'];
var InternalRemoteValues = ['INTERNAL', 'REMOTE'];
var OnOffValues = ['ON', 'OFF'];
var OperatingModeValues = ['OFF', 'AUTO', 'COOL', 'HEAT', 'EHEAT'];
var OperatingStatusValues = ['HEAT', 'COOL'];
var StageValues = ['0', '1', '2', '3'];
var TemperatureScaleValues = ['FAHRENHEIT', 'CELSIUS'];
var WaterValues = ['0', '1', 'OFF'];
var YesNoValues = ['YES', 'NO'];
var DayNightMap = new TSMap([
    ['NIGHT', false],
    ['DAY', true]
]);
var EnabledDisabledMap = new TSMap([
    ['DISABLED', false],
    ['ENABLED', true]
]);
var FanModeMap = new TSMap([
    ['AUTO', 0],
    ['ON', 1]
]);
var FanStatusMap = new TSMap([
    ['FAN AUTO', 0],
    ['FAN ON', 1],
    ['FAN RECIRC', 2]
]);
var InternalRemoteMap = new TSMap([
    ['INTERNAL', 0],
    ['REMOTE', 1]
]);
var OnOffMap = new TSMap([
    ['OFF', false],
    ['ON', true]
]);
var OperatingModeMap = new TSMap([
    ['OFF', 0],
    ['AUTO', 1],
    ['COOL', 2],
    ['HEAT', 3],
    ['EHEAT', 4]
]);
var OperatingStatusMap = new TSMap([
    ['HEAT', 0],
    ['COOL', 1]
]);
var StageMap = new TSMap([
    ['0', 0],
    ['1', 1],
    ['2', 2],
    ['3', 3]
]);
var TemperatureScaleMap = new TSMap([
    ['CELSIUS', 0],
    ['FAHRENHEIT', 1]
]);
var WaterMap = new TSMap([
    ['0', false],
    ['1', true],
    ['OFF', true]
]);
var YesNoMap = new TSMap([
    ['NO', false],
    ['YES', true]
]);
var ThermostatConnection = /** @class */ (function () {
    function ThermostatConnection(ipAddress, port, onCommRx, onConnect, onDisconnect, onTimeout, onReconnect, onConnectionStateChanged, logger) {
        this._logger = logger;
        this._loggerContext = 'ThermostatConnection (' + ipAddress + ':' + port + ')';
        logger.logTrace('constructor', this._loggerContext);
        this._ipAddress = ipAddress;
        this._port = port;
        this._tcp = new TCP(onCommRx);
        this._tcp.UseHandleInCallbacks = true;
        this._tcp.OnConnectFunc = onConnect;
        this._tcp.OnDisconnectFunc = onDisconnect;
        this._timeoutTimer = new Timer();
        this._timeoutTimer.UseHandleInCallbacks = true;
        this._reconnectTimer = new Timer();
        this._reconnectTimer.UseHandleInCallbacks = true;
        this._onTimeout = onTimeout;
        this._onReconnect = onReconnect;
        this._onConnectionStateChanged = onConnectionStateChanged;
        this.State = ConnectionState.Disconnected;
        this.TcpHandle = this._tcp.Handle;
        this.TimoutTimerHandle = this._timeoutTimer.Handle;
        this.ReconnectTimerHandle = this._reconnectTimer.Handle;
        logger.logTrace('Opening TCP', this._loggerContext);
        this._tcp.Open(ipAddress, port);
        this._tcp.AddRxFraming('StopChar', '\r\n');
        this._tcp.SetTxInterMsgDelay(100);
        this._timeoutTimer.Start(onTimeout, 5000);
    }
    ThermostatConnection.prototype.onConnect = function () {
        this._logger.logTrace('onConnect', this._loggerContext);
        this.setState(ConnectionState.Connected);
        if (this._timeoutTimer.State == 1) {
            this._timeoutTimer.Stop();
        }
        if (this._reconnectTimer.State == 1) {
            this._reconnectTimer.Stop();
        }
    };
    ThermostatConnection.prototype.onDisconnect = function () {
        this._logger.logTrace('onDisconnect', this._loggerContext);
        this.setState(ConnectionState.Disconnected);
        this._logger.logTrace('Disconnected, reconnecting', this._loggerContext);
        this.reconnect(5000);
    };
    ThermostatConnection.prototype.onTimeout = function () {
        this._logger.logTrace('onTimeout', this._loggerContext);
        this._logger.logTrace('TCP connection timed out, reconnecting', this._loggerContext);
        this.setState(ConnectionState.Failed);
        this.reconnect(5000);
    };
    ThermostatConnection.prototype.onReconnect = function () {
        this._logger.logTrace('onReconnect', this._loggerContext);
        this._tcp.Open(this._ipAddress, this._port);
        if (this._timeoutTimer.State == 1) {
            this._timeoutTimer.Stop();
        }
        this._timeoutTimer.Start(this._onTimeout, 5000);
    };
    ThermostatConnection.prototype.reconnect = function (delay) {
        if (delay === void 0) { delay = 0; }
        this._logger.logTrace('reconnect, delay: [' + delay + ']', this._loggerContext);
        if (this._reconnectTimer.State == 1) {
            this._reconnectTimer.Stop();
        }
        this.disconnect();
        this._logger.logTrace('starting reconnect timer', this._loggerContext);
        this._reconnectTimer.Start(this._onReconnect, delay);
    };
    ThermostatConnection.prototype.sendCommand = function (commandType, commandPayload) {
        if (commandPayload === void 0) { commandPayload = ''; }
        this._logger.logTrace('sendCommand (' + CommandType[commandType] + ', ' + commandPayload + ')', this._loggerContext);
        if (this.State != ConnectionState.Connected) {
            this._logger.logError('Cannot send command while disconnected.', this._loggerContext);
        }
        var commandClass = CommandType[commandType].charAt(0) == 'W' ? CommandClass.Write : CommandClass.Read;
        var commandPrefix = commandClass == CommandClass.Read ? CommandType[commandType] + '1' : CommandType[commandType] + '1D';
        var command = commandPrefix + commandPayload;
        this._logger.logTrace('command (' + command + ')', this._loggerContext);
        this._tcp.Write(command + '\r\n');
    };
    ThermostatConnection.prototype.sendRawCommand = function (command) {
        this._logger.logTrace('sendRawCommand (' + command + ')', this._loggerContext);
        this._tcp.Write(command + '\r\n');
    };
    ThermostatConnection.prototype.disconnect = function () {
        this._logger.logTrace('disconnect', this._loggerContext);
        if (this._tcp.OpenState == 1) {
            this._logger.logTrace('connection open, closing', this._loggerContext);
            this._tcp.Close();
        }
        this.setState(ConnectionState.Disconnected);
    };
    ThermostatConnection.prototype.setState = function (state) {
        this._logger.logTrace('setState, state: [' + ConnectionState[state] + ']', this._loggerContext);
        if (this.State == ConnectionState.Failed && state == ConnectionState.Disconnected) {
            // Failed state overrides Disconnected state
            this._logger.logTrace('Current connection state is faile, ignoring disconnected.', this._loggerContext);
            return;
        }
        if (state !== this.State) {
            this._logger.logTrace('Changing state to: [' + state + ']', this._loggerContext);
            this.State = state;
            this._onConnectionStateChanged(state);
        }
    };
    return ThermostatConnection;
}());
var Thermostat = /** @class */ (function () {
    function Thermostat(index, name, username, password, useSecureLogin, connection, onPollingIntervalElapsed, logger) {
        this.latestConnectionState = undefined;
        this._index = index;
        this.setNameValue(name);
        this._username = username;
        this._useSecureLogin = useSecureLogin;
        this.Connection = connection;
        this._onPollingIntervalElapsed = onPollingIntervalElapsed;
        this._logger = logger;
        this._loggerContext = 'Thermostat (' + name + ')';
        if (useSecureLogin) {
            var hashTarget = username + ':' + password;
            var hash = Crypto.Hash('SHA256', hashTarget, hashTarget.length);
            var base64Hash = Crypto.Base64Encode(hash, hash.length);
            this._password = base64Hash;
            this._logger.logTrace('hashTarget (' + hashTarget + ')', this._loggerContext);
            this._logger.logTrace('hash (' + hash + ')', this._loggerContext);
            this._logger.logTrace('base64Hash (' + base64Hash + ')', this._loggerContext);
        }
        else {
            this._password = password;
        }
        this._pollingTimer = new Timer();
        this._pollingTimer.UseHandleInCallbacks = true;
        this.PollingTimerHandle = this._pollingTimer.Handle;
    }
    Thermostat.prototype.setConnectedValue = function (value) { SystemVars.Write('Connected' + this._index, value); };
    Thermostat.prototype.setControlModeValue = function (value) { SystemVars.Write('ControlMode' + this._index, OnOffMap.get(value !== null && value !== void 0 ? value : 'OFF')); };
    Thermostat.prototype.setCoolSetpointMaximumValue = function (value) { SystemVars.Write('SPCoolHigh' + this._index, value || -999); };
    Thermostat.prototype.setCoolSetpointMinimumValue = function (value) { SystemVars.Write('SPCoolLow' + this._index, value || -999); };
    Thermostat.prototype.setCurrentCoolSetPointValue = function (value) { SystemVars.Write('CurrentCoolSetpoint' + this._index, value || -999); };
    Thermostat.prototype.setCurrentFanStatusValue = function (value) {
        var _a;
        SystemVars.Write('CurrentFan' + this._index, (_a = FanStatusMap.get(value)) !== null && _a !== void 0 ? _a : 3);
        SystemVars.Write('FanAuto' + this._index, value == 'FAN AUTO');
        SystemVars.Write('FanOn' + this._index, value == 'FAN ON');
        SystemVars.Write('FanRecirc' + this._index, value == 'FAN RECIRC');
    };
    Thermostat.prototype.setCurrentHeatSetPointValue = function (value) { SystemVars.Write('CurrentHeatSetpoint' + this._index, value || -999); };
    Thermostat.prototype.setCurrentOperatingModeValue = function (value) {
        var _a;
        SystemVars.Write('CurrentOperatingMode' + this._index, (_a = OperatingModeMap.get(value)) !== null && _a !== void 0 ? _a : 5);
        SystemVars.Write('CurrentOperatingModeOff' + this._index, value == 'OFF');
        SystemVars.Write('CurrentOperatingModeCool' + this._index, value == 'COOL');
        SystemVars.Write('CurrentOperatingModeHeat' + this._index, value == 'HEAT');
        SystemVars.Write('CurrentOperatingModeAuto' + this._index, value == 'AUTO');
    };
    Thermostat.prototype.setCurrentSetPointValue = function (value) { SystemVars.Write('CurrentSetpoint' + this._index, value || -999); };
    Thermostat.prototype.setDemandValue = function (value) { SystemVars.Write('Demand' + this._index, value !== null && value !== void 0 ? value : false); };
    Thermostat.prototype.setEmailProviderValue = function (value) { SystemVars.Write('EmailProvider' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setEventValue = function (value) { SystemVars.Write('Event' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setFanDemandValue = function (value) { SystemVars.Write('FanDemand' + this._index, value || false); };
    Thermostat.prototype.setFanModeValue = function (value) { var _a; SystemVars.Write('Fan' + this._index, (_a = FanModeMap.get(value)) !== null && _a !== void 0 ? _a : 2); };
    Thermostat.prototype.setHeatSetpointMaximumValue = function (value) { SystemVars.Write('SPHeatHigh' + this._index, value || -999); };
    Thermostat.prototype.setHeatSetpointMinimumValue = function (value) { SystemVars.Write('SPHeatLow' + this._index, value || -999); };
    Thermostat.prototype.setHoldOverrideUntilUnoccupiedValue = function (value) { SystemVars.Write('HoldOverrideUntilUnoccupied' + this._index, OnOffMap.get(value !== null && value !== void 0 ? value : 'OFF')); };
    Thermostat.prototype.setIndoorTempValue = function (value) { SystemVars.Write('IndoorTemp' + this._index, value || -999); };
    Thermostat.prototype.setNameValue = function (value) {
        this.Name = value !== null && value !== void 0 ? value : '';
        SystemVars.Write('Name' + this._index, value !== null && value !== void 0 ? value : '');
    };
    Thermostat.prototype.setOccupancyValue = function (value) { SystemVars.Write('Occupancy' + this._index, DayNightMap.get(value !== null && value !== void 0 ? value : 'DAY')); };
    Thermostat.prototype.setOccupiedCoolSetpointValue = function (value) { SystemVars.Write('OccupiedCoolSetpoint' + this._index, value || -999); };
    Thermostat.prototype.setOccupiedHeatSetpointValue = function (value) { SystemVars.Write('OccupiedHeatSetpoint' + this._index, value || -999); };
    Thermostat.prototype.setOperatingModeValue = function (value) { var _a; SystemVars.Write('OperatingMode' + this._index, (_a = OperatingModeMap.get(value)) !== null && _a !== void 0 ? _a : 5); };
    Thermostat.prototype.setOperatingStatusValue = function (value) { var _a; SystemVars.Write('OperatingStatus' + this._index, (_a = OperatingStatusMap.get(value)) !== null && _a !== void 0 ? _a : 2); };
    Thermostat.prototype.setOutdoorTempValue = function (value) { SystemVars.Write('OutdoorTemp' + this._index, value || -999); };
    Thermostat.prototype.setOverrideRemainingValue = function (value) { SystemVars.Write('OverrideRemaining' + this._index, value); };
    Thermostat.prototype.setOverrideTimeValue = function (value) { SystemVars.Write('OverrideTime' + this._index, value || -999); };
    Thermostat.prototype.setOverrideValue = function (value) { SystemVars.Write('Override' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteAux4TempValue = function (value) { SystemVars.Write('RemoteAux4Temp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteAux5TempValue = function (value) { SystemVars.Write('RemoteAux5Temp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteAux6TempValue = function (value) { SystemVars.Write('RemoteAux6Temp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteAux7TempValue = function (value) { SystemVars.Write('RemoteAux7Temp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteHumiditySourceValue = function (value) { var _a; SystemVars.Write('RemoteHumiditySource' + this._index, (_a = InternalRemoteMap.get(value)) !== null && _a !== void 0 ? _a : 2); };
    Thermostat.prototype.setRemoteHumidityValidValue = function (value) { SystemVars.Write('RemoteHumidityValid' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteHumidityValue = function (value) { SystemVars.Write('RemoteHumidity' + this._index, value || -999); };
    Thermostat.prototype.setRemoteIndoorAveragingValue = function (value) { SystemVars.Write('RemoteIndoorAveraging' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteIndoorTempValue = function (value) { SystemVars.Write('RemoteIndoorTemp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteIndoorValidValue = function (value) { SystemVars.Write('RemoteIndoorValid' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteOutdoorEnabledValue = function (value) { SystemVars.Write('RemoteOutdoorEnabled' + this._index, EnabledDisabledMap.get(value !== null && value !== void 0 ? value : 'DISABLED')); };
    Thermostat.prototype.setRemoteOutdoorTempValue = function (value) { SystemVars.Write('RemoteOutdoorTemp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteOutdoorValidValue = function (value) { SystemVars.Write('RemoteOutdoorValid' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteOccupancyValidValue = function (value) { SystemVars.Write('RemoteOccupancyValid' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteOccupancyValue = function (value) { SystemVars.Write('RemoteOccupancy' + this._index, value || -999); };
    Thermostat.prototype.setRemoteReturnTempValue = function (value) { SystemVars.Write('RemoteReturnTemp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteReturnValidValue = function (value) { SystemVars.Write('RemoteSpareValid' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteSpareTempValue = function (value) { SystemVars.Write('RemoteSpareTemp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteSpareValidValue = function (value) { SystemVars.Write('RemoteWaterValid' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteSupplyTempValue = function (value) { SystemVars.Write('RemoteSupplyTemp' + this._index, value || -999); };
    Thermostat.prototype.setRemoteSupplyValidValue = function (value) { SystemVars.Write('RemoteSupplyValid' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteWaterEnabledValue = function (value) { SystemVars.Write('RemoteWaterEnabled' + this._index, EnabledDisabledMap.get(value !== null && value !== void 0 ? value : 'DISABLED')); };
    Thermostat.prototype.setRemoteWaterValidValue = function (value) { SystemVars.Write('RemoteWaterValid' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setRemoteWaterValue = function (value) { SystemVars.Write('RemoteWater' + this._index, WaterMap.get(value !== null && value !== void 0 ? value : '0')); };
    Thermostat.prototype.setSiteNameValue = function (value) { SystemVars.Write('SiteName' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setSiteAddressValue = function (value) { SystemVars.Write('SiteAddress' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setSiteCityValue = function (value) { SystemVars.Write('SiteCity' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setSiteStateValue = function (value) { SystemVars.Write('SiteState' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setSitePostalCodeValue = function (value) { SystemVars.Write('SitePostalCode' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setSiteCountryValue = function (value) { SystemVars.Write('SiteCountry' + this._index, value !== null && value !== void 0 ? value : 'US'); };
    Thermostat.prototype.setSiteContactValue = function (value) { SystemVars.Write('SiteContact' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setRecirculationStatusValue = function (value) { SystemVars.Write('RecirculationStatus' + this._index, OnOffMap.get(value !== null && value !== void 0 ? value : 'OFF')); };
    Thermostat.prototype.setRecirculationTimerValue = function (value) { SystemVars.Write('RecirculationTimer' + this._index, value || -999); };
    Thermostat.prototype.setRecoveryValue = function (value) { SystemVars.Write('Recovery' + this._index, YesNoMap.get(value !== null && value !== void 0 ? value : 'NO')); };
    Thermostat.prototype.setSmtpAuthUsernameValue = function (value) { SystemVars.Write('SmtpAuthUsername' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setSmtpAuthPasswordValue = function (value) { SystemVars.Write('SmtpAuthPassword' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setSmtpServerAddressValue = function (value) { SystemVars.Write('SmtpAddress' + this._index, value !== null && value !== void 0 ? value : ''); };
    Thermostat.prototype.setSmtpPortValue = function (value) { SystemVars.Write('SmtpPort' + this._index, value | -999); };
    Thermostat.prototype.setSmtpSslValue = function (value) { SystemVars.Write('SmtpSsl' + this._index, OnOffMap.get(value !== null && value !== void 0 ? value : 'OFF')); };
    Thermostat.prototype.setStageValue = function (value) { var _a; SystemVars.Write('Stage' + this._index, (_a = StageMap.get(value)) !== null && _a !== void 0 ? _a : 4); };
    Thermostat.prototype.setTemperatureScaleValue = function (value) { var _a; SystemVars.Write('TemperatureScale' + this._index, (_a = TemperatureScaleMap.get(value)) !== null && _a !== void 0 ? _a : 2); };
    Thermostat.prototype.setUnoccupiedCoolSetpointValue = function (value) { SystemVars.Write('UnoccupiedCoolSetpoint' + this._index, value || -999); };
    Thermostat.prototype.setUnoccupiedHeatSetpointValue = function (value) { SystemVars.Write('UnoccupiedHeatSetpoint' + this._index, value || -999); };
    Thermostat.prototype.authorize = function (salt) {
        this._logger.logTrace('authorize', this._loggerContext);
        this._logger.logTrace('salt (' + salt + ')', this._loggerContext);
        if (this._useSecureLogin) {
            if (salt) {
                var hashTarget = salt + '$' + this._password;
                var hash = Crypto.Hash('SHA256', hashTarget, hashTarget.length);
                var base64Hash = Crypto.Base64Encode(hash, hash.length);
                this._logger.logTrace('hashTarget (' + hashTarget + ')', this._loggerContext);
                this._logger.logTrace('hash (' + hash + ')', this._loggerContext);
                this._logger.logTrace('base64Hash (' + base64Hash + ')', this._loggerContext);
                this.Connection.sendCommand(CommandType.WMLS, this._username + ',' + base64Hash);
            }
            else {
                this.Connection.sendCommand(CommandType.RMLS);
            }
        }
        else {
            this.Connection.sendCommand(CommandType.WML, this._username + ',' + this._password);
        }
    };
    Thermostat.prototype.calculateDemand = function (mode, stage) {
        this._logger.logTrace('calculateDemand', this._loggerContext);
        if (!mode || !stage || mode == 'OFF') {
            return false;
        }
        return stage !== '0';
    };
    Thermostat.prototype.calculateFanDemand = function (fanStatus, stage) {
        this._logger.logTrace('calculateFanDemand', this._loggerContext);
        if (!fanStatus || !stage) {
            return false;
        }
        if (fanStatus == 'FAN ON') {
            return true;
        }
        return stage !== '0';
    };
    Thermostat.prototype.calculateCurrentSetpoint = function (operatingStatus, coolSetpoint, heatSetpoint) {
        this._logger.logTrace('calculateCurrentOperatingMode', this._loggerContext);
        this._logger.logTrace('operatingStatus (' + operatingStatus + ')', this._loggerContext);
        this._logger.logTrace('coolSetpoint (' + coolSetpoint + ')', this._loggerContext);
        this._logger.logTrace('heatSetpoint (' + heatSetpoint + ')', this._loggerContext);
        if (!operatingStatus) {
            return NaN;
        }
        if (operatingStatus == 'COOL') {
            return coolSetpoint;
        }
        if (operatingStatus == 'HEAT') {
            return heatSetpoint;
        }
        return NaN;
    };
    Thermostat.prototype.clearStatus = function () {
        this._logger.logTrace('clearStatus', this._loggerContext);
        this.setControlModeValue(undefined);
        this.setCoolSetpointMaximumValue(NaN);
        this.setCoolSetpointMinimumValue(NaN);
        this.setCurrentCoolSetPointValue(NaN);
        this.setCurrentFanStatusValue(undefined);
        this.setCurrentHeatSetPointValue(NaN);
        this.setCurrentOperatingModeValue(undefined);
        this.setCurrentSetPointValue(NaN);
        this.setDemandValue(undefined);
        this.setEmailProviderValue(undefined);
        this.setEventValue(undefined);
        this.setFanDemandValue(undefined);
        this.setFanModeValue(undefined);
        this.setHeatSetpointMaximumValue(NaN);
        this.setHeatSetpointMinimumValue(NaN);
        this.setHoldOverrideUntilUnoccupiedValue(undefined);
        this.setIndoorTempValue(NaN);
        this.setOccupancyValue(undefined);
        this.setOccupiedCoolSetpointValue(NaN);
        this.setOccupiedHeatSetpointValue(NaN);
        this.setOperatingModeValue(undefined);
        this.setOccupancyValue(undefined);
        this.setOccupiedCoolSetpointValue(NaN);
        this.setOccupiedHeatSetpointValue(NaN);
        this.setOperatingStatusValue(undefined);
        this.setOutdoorTempValue(NaN);
        this.setOverrideTimeValue(NaN);
        this.setOverrideValue(undefined);
        this.setRecirculationStatusValue(undefined);
        this.setRecirculationTimerValue(NaN);
        this.setRecoveryValue(undefined);
        this.setSiteAddressValue(undefined);
        this.setSiteCityValue(undefined);
        this.setSiteContactValue(undefined);
        this.setSiteCountryValue(undefined);
        this.setSiteNameValue(undefined);
        this.setSitePostalCodeValue(undefined);
        this.setSiteStateValue(undefined);
        this.setSmtpAuthPasswordValue(undefined);
        this.setSmtpAuthUsernameValue(undefined);
        this.setSmtpPortValue(NaN);
        this.setSmtpServerAddressValue(undefined);
        this.setSmtpSslValue(undefined);
        this.setStageValue(undefined);
        this.setTemperatureScaleValue(undefined);
    };
    Thermostat.prototype.getConfiguration = function () {
        this._logger.logTrace('getConfiguration', this._loggerContext);
        this.Connection.sendCommand(CommandType.RMTN);
        this.Connection.sendCommand(CommandType.RTS);
        this.Connection.sendCommand(CommandType.RCL);
        this.Connection.sendCommand(CommandType.RCH);
        this.Connection.sendCommand(CommandType.RHL);
        this.Connection.sendCommand(CommandType.RHH);
        this.Connection.sendCommand(CommandType.ROU);
        this.Connection.sendCommand(CommandType.ROP);
        this.Connection.sendCommand(CommandType.RRT);
        this.Connection.sendCommand(CommandType.RMES);
        this.Connection.sendCommand(CommandType.RMESD);
        this.Connection.sendCommand(CommandType.RMTST);
    };
    Thermostat.prototype.overrideCurrentSetpoint = function (adjustment) {
        var _a;
        this._logger.logTrace('adjustCurrentSetpoint', this._loggerContext);
        this._logger.logTrace('adjustment (' + adjustment + ')', this._loggerContext);
        var operatingMode = (_a = OperatingModeMap.getKey(parseInt(SystemVars.Read('CurrentOperatingMode' + this._index)))) !== null && _a !== void 0 ? _a : 'OFF';
        var operatingStatus = OperatingStatusMap.getKey(parseInt(SystemVars.Read('OperatingStatus' + this._index)));
        var initialCoolSetpoint = parseInt(SystemVars.Read('CurrentCoolSetpoint' + this._index), 10);
        var initialHeatSetpoint = parseInt(SystemVars.Read('CurrentHeatSetpoint' + this._index), 10);
        this._logger.logTrace('operatingMode (' + operatingMode + ')', this._loggerContext);
        this._logger.logTrace('operatingStatus (' + operatingStatus + ')', this._loggerContext);
        if (operatingMode == 'COOL' || (operatingMode == 'AUTO' && operatingStatus == 'COOL')) {
            var updatedCoolSetpoint = initialCoolSetpoint + adjustment;
            this._logger.logTrace('initialCoolSetpoint (' + initialCoolSetpoint + ')', this._loggerContext);
            this._logger.logTrace('updatedCoolSetpoint (' + updatedCoolSetpoint + ')', this._loggerContext);
            this.setCurrentCoolSetPointValue(updatedCoolSetpoint);
            this.setCurrentSetPointValue(this.calculateCurrentSetpoint(operatingStatus, updatedCoolSetpoint, initialHeatSetpoint));
            this.Connection.sendCommand(CommandType.WMS, operatingMode); // Used to trigger override with default time remaining
            this.Connection.sendCommand(CommandType.WOC, updatedCoolSetpoint.toString()); // Write Override Cool
        }
        if (operatingMode == 'EHEAT' || operatingMode == 'HEAT' || (operatingMode == 'AUTO' && operatingStatus == 'HEAT')) {
            var updatedHeatSetpoint = initialHeatSetpoint + adjustment;
            this.setCurrentHeatSetPointValue(updatedHeatSetpoint);
            this.setCurrentSetPointValue(this.calculateCurrentSetpoint(operatingStatus, initialCoolSetpoint, updatedHeatSetpoint));
            this.Connection.sendCommand(CommandType.WMS, operatingMode); // Used to trigger override with default time remaining
            this.Connection.sendCommand(CommandType.WOH, updatedHeatSetpoint.toString()); // Write Override Heat
        }
        this.refreshStatus();
    };
    Thermostat.prototype.refreshStatus = function () {
        this._logger.logTrace('refreshStatus', this._loggerContext);
        this.Connection.sendCommand(CommandType.RAS); // Read All States
        this.Connection.sendCommand(CommandType.RRIA); // Read Remote Indoor Averaging
        this.Connection.sendCommand(CommandType.RRIV); // Read Remote Indoor Validity
        this.Connection.sendCommand(CommandType.RROV); // Read Remote Outdoor Validity
        this.Connection.sendCommand(CommandType.RROE); // Read Remote Outdoor Enable
        this.Connection.sendCommand(CommandType.RRHSS); // Read Remote Humidity Sensor Source
        this.Connection.sendCommand(CommandType.RRHV); // Read Remote Humidity Validity
        this.Connection.sendCommand(CommandType.RRSV); // Read Remote Supply Validity
        this.Connection.sendCommand(CommandType.RRRV); // Read Remote Return Validity
        this.Connection.sendCommand(CommandType.RRXV); // Read Remote Spare Validity
        this.Connection.sendCommand(CommandType.RRPV); // Read Remote People (Occupancy) Validity
        this.Connection.sendCommand(CommandType.RRWV); // Read Remote Water Validity
        this.Connection.sendCommand(CommandType.RRWE); // Read Remote Water Enable
        this.Connection.sendCommand(CommandType.RRA); // Read Remote All
        this.Connection.sendCommand(CommandType.RRS); // Read Recirculation Status
        this.Connection.sendCommand(CommandType.RNS); // Read Non-programmable (Manual) Status
        this.Connection.sendCommand(CommandType.RNDS); // Read Non-programmable Day-Night (Occ/Unocc) Status
        this.Connection.sendCommand(CommandType.RNFM); // Read Non-programmable Fan Mode Status
        this.Connection.sendCommand(CommandType.RNMS); // Read Non-programmable Mode Status
        this.Connection.sendCommand(CommandType.ROR); // Read Override Remaining
    };
    Thermostat.prototype.writeConfiguration = function () {
        this._logger.logTrace('writeConfiguration', this._loggerContext);
        var tempScale = Config.Get('TempScale' + this._index);
        var coolSetpointMin = Config.Get('SPCoolMin' + this._index);
        var coolSetpointMax = Config.Get('SPCoolMax' + this._index);
        var heatSetpointMin = Config.Get('SPHeatMin' + this._index);
        var heatSetpointMax = Config.Get('SPHeatMax' + this._index);
        var holdUntilUnoccupied = Config.Get('HoldUntilUnoccupied' + this._index) == 'true';
        var overrideTime = parseInt(Config.Get('OverrideTime' + this._index), 10);
        var recirculationTime = parseInt(Config.Get('RecirculationTime' + this._index), 10);
        var useCustomSmtp = Config.Get('UseCustomSmtp') == 'true';
        var smtpServerAddress = Config.Get('SmtpAddress');
        var smtpPort = parseInt(Config.Get('SmtpPort'), 10);
        var smtpSSL = OnOffMap.getKey(Config.Get('SmtpSSL') == 'true');
        var smtpAuthUseraname = Config.Get('SmtpAuthUsername');
        var smtpAuthPassword = Config.Get('SmtpAuthPassword');
        var siteName = Config.Get('SiteName');
        var siteAddress = Config.Get('SiteAddress');
        var siteCity = Config.Get('SiteCity');
        var siteState = Config.Get('SiteState');
        var sitePostalCode = Config.Get('SitePostalCode');
        var siteCountry = Config.Get('SiteCountry');
        var siteContact = Config.Get('SiteContact');
        this.Connection.sendCommand(CommandType.WMTN, this.Name);
        this.Connection.sendCommand(CommandType.WTS, tempScale);
        this.Connection.sendCommand(CommandType.WCL, coolSetpointMin);
        this.Connection.sendCommand(CommandType.WCH, coolSetpointMax);
        this.Connection.sendCommand(CommandType.WHL, heatSetpointMin);
        this.Connection.sendCommand(CommandType.WHH, heatSetpointMax);
        this.Connection.sendCommand(CommandType.WOU, OnOffMap.getKey(holdUntilUnoccupied));
        if (!holdUntilUnoccupied) {
            this.Connection.sendCommand(CommandType.WOP, overrideTime.toString());
        }
        this.Connection.sendCommand(CommandType.WRT, recirculationTime.toString());
        if (useCustomSmtp) {
            this.Connection.sendCommand(CommandType.WMES, [smtpAuthUseraname, smtpAuthPassword, smtpServerAddress, smtpPort, smtpSSL].join(','));
        }
        else {
            this.Connection.sendCommand(CommandType.WMESD, 'DEFAULT');
        }
        this.Connection.sendCommand(CommandType.WMTST, [siteName, siteAddress, siteCity, siteState, sitePostalCode, siteCountry, siteContact].join(','));
    };
    Thermostat.prototype.DecrementCurrentSetpoint = function () {
        this._logger.logTrace('decrementCurrentSetpoint', this._loggerContext);
        this.overrideCurrentSetpoint(-1);
    };
    Thermostat.prototype.IncrementCurrentSetpoint = function () {
        this._logger.logTrace('incrementCurrentSetpoint', this._loggerContext);
        this.overrideCurrentSetpoint(1);
    };
    Thermostat.prototype.OnCommRx = function (data) {
        data = data.replace(/[\r\n]/gm, ''); // remove line breaks
        this._logger.logTrace('onCommRx', this._loggerContext);
        this._logger.logTrace('data (' + data + ')', this._loggerContext);
        // Handle successful login
        if (data.indexOf('OK,USER') == 0) {
            // OK,USER,NO
            this._logger.logInfo('Successful Login', this._loggerContext);
            this.writeConfiguration();
            this.getConfiguration();
            this.refreshStatus();
            this._pollingTimer.Start(this._onPollingIntervalElapsed, 10000);
            System.SignalEvent('Initialized' + this._index);
            return;
        }
        var isCommand = (data[0] == 'R' || data[0] == 'W') && data.indexOf('1') > 0;
        if (!isCommand) {
            this._logger.logTrace(data, this._loggerContext);
            return;
        }
        var commandClass = data[0] == 'R' ? CommandClass.Read : CommandClass.Write;
        var commandDelimiter = commandClass == CommandClass.Read ? '1:' : '1D';
        var commandType = CommandType[data.substring(0, data.indexOf(commandDelimiter))];
        var commandPayload = data.substring(data.indexOf(commandDelimiter) + commandDelimiter.length);
        this._logger.logTrace('commandClass (' + CommandClass[commandClass] + '), commandType (' + CommandType[commandType] + '), commandPayload (' + commandPayload + ')', this._loggerContext);
        switch (commandType) {
            case CommandType.RAS: {
                // RAS1:indoor,outdoor,mode,fan,override,recovery,spcool,spheat,opstatus,stage,event
                var values_1 = commandPayload.split(',');
                var operatingMode = OperatingModeValues.some(function (x) { return x == values_1[2]; }) ? values_1[2] : undefined;
                var stage = StageValues.some(function (x) { return x == values_1[9]; }) ? values_1[9] : undefined;
                var fanStatus = FanStatusValues.some(function (x) { return x == values_1[3]; }) ? values_1[3] : undefined;
                var operatingStatus = OperatingStatusValues.some(function (x) { return x == values_1[8]; }) ? values_1[8] : undefined;
                var coolSetpoint = parseInt(values_1[6], 10);
                var heatSetpoint = parseInt(values_1[7], 10);
                this.setCurrentOperatingModeValue(operatingMode);
                this.setCurrentCoolSetPointValue(coolSetpoint);
                this.setCurrentFanStatusValue(fanStatus);
                this.setCurrentHeatSetPointValue(heatSetpoint);
                this.setCurrentSetPointValue(this.calculateCurrentSetpoint(operatingStatus, coolSetpoint, heatSetpoint));
                this.setDemandValue(this.calculateDemand(operatingMode, stage));
                this.setEventValue(values_1[10]);
                this.setFanDemandValue(this.calculateFanDemand(fanStatus, stage));
                this.setIndoorTempValue(parseInt(values_1[0], 10));
                this.setOperatingStatusValue(operatingStatus);
                this.setOutdoorTempValue(parseInt(values_1[1], 10));
                this.setOverrideValue(YesNoValues.some(function (x) { return x == values_1[4]; }) ? values_1[4] : undefined);
                this.setStageValue(stage);
                this.setRecoveryValue(YesNoValues.some(function (x) { return x == values_1[5]; }) ? values_1[5] : undefined);
                break;
            }
            case CommandType.RCH:
                // RCH1:temp
                this.setCoolSetpointMaximumValue(parseInt(commandPayload, 10));
                break;
            case CommandType.RCL:
                // RCL1:temp
                this.setCoolSetpointMinimumValue(parseInt(commandPayload, 10));
                break;
            case CommandType.RHH:
                // RHH1:temp
                this.setHeatSetpointMaximumValue(parseInt(commandPayload, 10));
                break;
            case CommandType.RHL:
                // RCL1:temp
                this.setHeatSetpointMinimumValue(parseInt(commandPayload, 10));
                break;
            case CommandType.RMES: {
                // RMES1:smtpAuthUseraname,smtpAuthPassword,smtpServerAddress,smtpPort,smtpSSL
                var values_2 = commandPayload.split(',');
                var smtpAuthUsername = values_2[0];
                var smtpAuthPassword = values_2[1];
                var smtpServerAddress = values_2[2];
                var smtpPort = parseInt(values_2[3], 10);
                var smtpSSL = OnOffValues.some(function (x) { return x == values_2[4]; }) ? values_2[4] : undefined;
                this.setSmtpAuthUsernameValue(smtpAuthUsername);
                this.setSmtpAuthPasswordValue(smtpAuthPassword);
                this.setSmtpServerAddressValue(smtpServerAddress);
                this.setSmtpPortValue(smtpPort);
                this.setSmtpSslValue(smtpSSL);
                break;
            }
            case commandType.RMESD:
                // RMESD1:emailprovider
                this.setEmailProviderValue(commandPayload);
                break;
            case CommandType.RMLS:
                // RMLS1:loginsalt
                this.authorize(commandPayload);
                break;
            case CommandType.RMTN:
                // RMTN1:name
                this.setNameValue(commandPayload);
                break;
            case CommandType.RMTST: {
                // RMTST1:sitename,siteaddress,sitecity,sitestate,sitezipcode,sitecountry,sitecontact
                var values_3 = commandPayload.split(',');
                var siteName = values_3[0];
                var siteAddress = values_3[1];
                var siteCity = values_3[2];
                var siteState = values_3[3];
                var sitePostalCode = values_3[4];
                var siteCountry = CountryValues.some(function (x) { return x == values_3[5]; }) ? values_3[5] : undefined;
                var siteContact = values_3[6];
                this.setSiteNameValue(siteName);
                this.setSiteAddressValue(siteAddress);
                this.setSiteCityValue(siteCity);
                this.setSiteStateValue(siteState);
                this.setSitePostalCodeValue(sitePostalCode);
                this.setSiteCountryValue(siteCountry);
                this.setSiteContactValue(siteContact);
                break;
            }
            case CommandType.RNDS:
                // RNDS1:manualdaynight
                this.setOccupancyValue(DayNigthValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RNS:
                // RNS1:manualstatus
                this.setControlModeValue(OnOffValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RNFM:
                // RNFM1:FAN manualfan
                this.setFanModeValue(FanModeValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RNMS:
                // RNMS1:manualmode
                this.setOperatingModeValue(OperatingModeValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.ROP:
                // ROP1:overridetime
                this.setOverrideTimeValue(parseInt(commandPayload, 10));
                break;
            case CommandType.ROR:
                // ROR1:overrideremaining
                this.setOverrideRemainingValue(commandPayload);
                break;
            case CommandType.ROU:
                // ROU:holduntilunoccupied
                this.setHoldOverrideUntilUnoccupiedValue(OnOffValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRA: {
                // RA1:remoteindoor,remoteoutdoor,remotehumidity,remotesupply,remotereturn,remotespare,remoteoccupancy,remouteaux4,remouteaux6,remouteaux6,remouteaux7,remotewater
                var values_4 = commandPayload.split(',');
                var remoteIndoorTemp = parseInt(values_4[0], 10);
                var remoteOutdoorTemp = parseInt(values_4[1], 10);
                var remoteHumidity = parseInt(values_4[2], 10);
                var remoteSupplyTemp = parseInt(values_4[3], 10);
                var remoteReturnTemp = parseInt(values_4[4], 10);
                var remoteSpareTemp = parseInt(values_4[5], 10);
                var remoteOccupancy = parseInt(values_4[6], 10);
                var remoteAux4Temp = parseInt(values_4[7], 10);
                var remoteAux5Temp = parseInt(values_4[8], 10);
                var remoteAux6Temp = parseInt(values_4[9], 10);
                var remoteAux7Temp = parseInt(values_4[10], 10);
                var remoteWater = WaterValues.some(function (x) { return x == values_4[11]; }) ? values_4[11] : undefined;
                this.setRemoteIndoorTempValue(remoteIndoorTemp);
                this.setRemoteOutdoorTempValue(remoteOutdoorTemp);
                this.setRemoteHumidityValue(remoteHumidity);
                this.setRemoteSupplyTempValue(remoteSupplyTemp);
                this.setRemoteReturnTempValue(remoteReturnTemp);
                this.setRemoteSpareTempValue(remoteSpareTemp);
                this.setRemoteOccupancyValue(remoteOccupancy);
                this.setRemoteAux4TempValue(remoteAux4Temp);
                this.setRemoteAux5TempValue(remoteAux5Temp);
                this.setRemoteAux6TempValue(remoteAux6Temp);
                this.setRemoteAux7TempValue(remoteAux7Temp);
                this.setRemoteWaterValue(remoteWater);
                break;
            }
            case CommandType.RRHSS:
                // RRHSS1:remotehumiditysource
                this.setRemoteHumiditySourceValue(InternalRemoteValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRHV:
                // RRHV1:remotehumidityvalid
                this.setRemoteHumidityValidValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRIA:
                // RRIA1:remoteindooravg
                this.setRemoteIndoorAveragingValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRIV:
                // RRIV1:remoteindoorvalid
                this.setRemoteIndoorValidValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RROE:
                // RROE1:remoteoutdoorenabled
                this.setRemoteOutdoorEnabledValue(EnabledDisabledValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RROV:
                // RROV1:remoteoutdoorvalid
                this.setRemoteOutdoorValidValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRPV:
                // RRPV1:remoteoccupancyvalid
                this.setRemoteOccupancyValidValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRRV:
                // RRRV1:remotereturnvalid
                this.setRemoteReturnValidValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRS:
                // RRS1:recircstatus
                this.setRecirculationStatusValue(OnOffValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRSV:
                // RRSV1:remotesupplyvalid
                this.setRemoteSupplyValidValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRT:
                // RRT1:recirctimer
                this.setRecirculationTimerValue(parseInt(commandPayload, 10));
                break;
            case CommandType.RRWE:
                // RRWE1:remotewaterenable
                this.setRemoteWaterEnabledValue(EnabledDisabledValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRWV:
                // RRWV1:remotewatervalid
                this.setRemoteWaterValidValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RRXV:
                // RRXV1:remotesparevalid
                this.setRemoteSpareValidValue(YesNoValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            case CommandType.RTS:
                // RTS1:scale
                this.setTemperatureScaleValue(TemperatureScaleValues.some(function (x) { return x == commandPayload; }) ? commandPayload : undefined);
                break;
            default:
                break;
        }
    };
    Thermostat.prototype.OnConnectionStateChanged = function (state) {
        this._logger.logTrace('onConnectionStateChanged', this._loggerContext);
        this._logger.logTrace('state (' + ConnectionState[state] + ')', this._loggerContext);
        if (state == this.latestConnectionState) {
            return;
        }
        this.latestConnectionState = state;
        switch (state) {
            case ConnectionState.Connected:
                this.setConnectedValue(true);
                System.SignalEvent('Connected' + this._index);
                this.authorize();
                break;
            case ConnectionState.Disconnected:
                this.setConnectedValue(false);
                System.SignalEvent('Disconnected' + this._index);
                this._pollingTimer.Stop();
                this.clearStatus();
                break;
            case ConnectionState.Failed:
                this.setConnectedValue(false);
                System.SignalEvent('ConnectionFailed' + this._index);
                break;
        }
    };
    Thermostat.prototype.OnPollingIntervalElapsed = function () {
        this._logger.logTrace('onPollingIntervalElapsed', this._loggerContext);
        this._pollingTimer.Start(this._onPollingIntervalElapsed, 10000);
        this.refreshStatus();
    };
    Thermostat.prototype.OverrideFanMode = function (overrideFanMode) {
        this._logger.logTrace('OverrideFanMode', this._loggerContext);
        this._logger.logTrace('fanMode (' + overrideFanMode + ')', this._loggerContext);
        this.setCurrentFanStatusValue(overrideFanMode == 'AUTO' ? 'FAN AUTO' : 'FAN ON');
        this.Connection.sendCommand(CommandType.WFM, overrideFanMode); // Write Fan Mode
        this.refreshStatus();
    };
    Thermostat.prototype.OverrideOperatingMode = function (opderatingMode) {
        this._logger.logTrace('OverrideOperatingMode', this._loggerContext);
        this._logger.logTrace('opderatingMode (' + opderatingMode + ')', this._loggerContext);
        this.setCurrentOperatingModeValue(opderatingMode);
        this.Connection.sendCommand(CommandType.WMS, opderatingMode); // Write Mode Status
        this.refreshStatus();
    };
    Thermostat.prototype.SendRawCommand = function (command) {
        this._logger.logTrace('SendRawCommand', this._loggerContext);
        this._logger.logTrace('command (' + command + ')', this._loggerContext);
        this.Connection.sendRawCommand(command);
    };
    Thermostat.prototype.SetControlMode = function (manualStatus) {
        this._logger.logTrace('SetControlMode', this._loggerContext);
        this._logger.logTrace('manualStatus (' + manualStatus + ')', this._loggerContext);
        this.setControlModeValue(manualStatus);
        this.Connection.sendCommand(CommandType.WNS, manualStatus == 'ON' ? '1' : '0');
    };
    Thermostat.prototype.SetFanMode = function (fanMode) {
        this._logger.logTrace('SetFanMode', this._loggerContext);
        this._logger.logTrace('fanMode (' + fanMode + ')', this._loggerContext);
        this.setFanModeValue(fanMode);
        this.Connection.sendCommand(CommandType.WNFM, fanMode); // Write Non-Programmable Fan Mode
        this.refreshStatus();
    };
    Thermostat.prototype.SetOccupancy = function (state) {
        this._logger.logTrace('SetOccupied', this._loggerContext);
        this._logger.logTrace('state (' + state + ')', this._loggerContext);
        this.setOccupancyValue(state);
        this.Connection.sendCommand(CommandType.WNDS, state.toString());
    };
    Thermostat.prototype.SetOccupiedCoolSetpoint = function (temperature) {
        this._logger.logTrace('SetOccupiedCoolSetpoint', this._loggerContext);
        this._logger.logTrace('temperature (' + temperature + ')', this._loggerContext);
        this.setOccupiedCoolSetpointValue(temperature);
        this.Connection.sendCommand(CommandType.WNCD, temperature.toString());
    };
    Thermostat.prototype.SetOccupiedHeatSetpoint = function (temperature) {
        this._logger.logTrace('SetOccupiedHeatSetpoint', this._loggerContext);
        this._logger.logTrace('temperature (' + temperature + ')', this._loggerContext);
        this.setOccupiedHeatSetpointValue(temperature);
        this.Connection.sendCommand(CommandType.WNHD, temperature.toString());
    };
    Thermostat.prototype.SetOperatingMode = function (opderatingMode) {
        this._logger.logTrace('SetOperatingMode', this._loggerContext);
        this._logger.logTrace('opderatingMode (' + opderatingMode + ')', this._loggerContext);
        this.setOperatingModeValue(opderatingMode);
        this.Connection.sendCommand(CommandType.WNMS, opderatingMode); // Write Non-Programmable Mode Status
        this.refreshStatus();
    };
    Thermostat.prototype.SetOverrideRemaining = function (hours, minutes) {
        this._logger.logTrace('SetOverrideRemaining', this._loggerContext);
        this._logger.logTrace('hours (' + hours + ')', this._loggerContext);
        this._logger.logTrace('minutes (' + minutes + ')', this._loggerContext);
        var timeRemaining = hours + ':' + minutes;
        this.setOverrideRemainingValue(timeRemaining);
        this.Connection.sendCommand(CommandType.WOR, timeRemaining); // Write Override Remaining
        this.refreshStatus();
    };
    Thermostat.prototype.SetUnoccupiedCoolSetpoint = function (temperature) {
        this._logger.logTrace('SetUnoccupiedCoolSetpoint', this._loggerContext);
        this._logger.logTrace('temperature (' + temperature + ')', this._loggerContext);
        this.setUnoccupiedCoolSetpointValue(temperature);
        this.Connection.sendCommand(CommandType.WNCN, temperature.toString());
    };
    Thermostat.prototype.SetUnoccupiedHeatSetpoint = function (temperature) {
        this._logger.logTrace('SetUnoccupiedHeatSetpoint', this._loggerContext);
        this._logger.logTrace('temperature (' + temperature + ')', this._loggerContext);
        this.setUnoccupiedHeatSetpointValue(temperature);
        this.Connection.sendCommand(CommandType.WNHN, temperature.toString());
    };
    return Thermostat;
}());
var g_debug = Config.Get('DebugTrace') == 'true';
var g_logger = new Logger('Network Thermostat Driver', g_debug);
g_logger.logInfo('Initializing Network Thermostat Driver');
var g_totalThermostatCount = parseInt(Config.Get('TotalThermostatCount'));
var g_thermostats = new Array();
var g_thermostatGlobalHandlerMap = new GlobalHandlerMap();
var _loop_1 = function (i) {
    var name = Config.Get('ThermostatName' + i).replace(' ', '_');
    var address = Config.Get('ThermostatAddress' + i);
    var port = parseInt(Config.Get('ThermostatPort' + i), 10);
    var username = Config.Get('AuthUsername' + i);
    var password = Config.Get('AuthPassword' + i);
    var useSecureLogin = Config.Get('UseSecureLogin' + i) == 'true';
    var thermostat = new Thermostat(i, name, username, password, useSecureLogin, new ThermostatConnection(address, port, ThermostatOnCommRx, ThermostatConnectionOnConnect, ThermostatConnectionOnDisconnect, ThermostatConnectionOnTimeout, ThermostatConnectionOnReconnect, function (state) { thermostat.OnConnectionStateChanged(state); }, g_logger), ThermostatOnPollingIntervalElapsed, g_logger);
    g_thermostatGlobalHandlerMap.register(thermostat.PollingTimerHandle, thermostat);
    g_thermostatGlobalHandlerMap.register(thermostat.Connection.TcpHandle, thermostat);
    g_thermostatGlobalHandlerMap.register(thermostat.Connection.TimoutTimerHandle, thermostat);
    g_thermostatGlobalHandlerMap.register(thermostat.Connection.ReconnectTimerHandle, thermostat);
    g_thermostats[i] = thermostat;
};
for (var i = 1; i <= g_totalThermostatCount; i++) {
    _loop_1(i);
}
//#region ThermostatConnection event handlers
function ThermostatConnectionOnConnect(handle) {
    g_logger.logTrace('ThermostatConnectionOnConnect');
    var thermostat = g_thermostatGlobalHandlerMap.getMappedValueFromHandle(handle);
    if (!thermostat) {
        g_logger.logError('ThermostatConnectionOnConnect: Error retrieving device from handle: ' + handle);
        return;
    }
    thermostat.Connection.onConnect();
}
function ThermostatConnectionOnDisconnect(handle) {
    g_logger.logTrace('ThermostatConnectionOnDisconnect');
    var thermostat = g_thermostatGlobalHandlerMap.getMappedValueFromHandle(handle);
    if (!thermostat) {
        g_logger.logError('ThermostatConnectionOnDisconnect: Error retrieving device from handle: ' + handle);
        return;
    }
    thermostat.Connection.onDisconnect();
}
function ThermostatConnectionOnTimeout(handle) {
    g_logger.logTrace('ThermostatConnectionOnTimeout');
    var thermostat = g_thermostatGlobalHandlerMap.getMappedValueFromHandle(handle);
    if (!thermostat) {
        g_logger.logError('ThermostatConnectionOnTimeout: Error retrieving device from handle: ' + handle);
        return;
    }
    thermostat.Connection.onTimeout();
}
function ThermostatConnectionOnReconnect(handle) {
    g_logger.logTrace('ThermostatConnectionOnReconnect');
    var thermostat = g_thermostatGlobalHandlerMap.getMappedValueFromHandle(handle);
    if (!thermostat) {
        g_logger.logError('ThermostatConnectionOnReconnect: Error retrieving device from handle: ' + handle);
        return;
    }
    thermostat.Connection.onReconnect();
}
//#endregion
//#region Thermostat event handlers
function ThermostatOnCommRx(data, handle) {
    g_logger.logTrace('ThermostatOnCommRx');
    var thermostat = g_thermostatGlobalHandlerMap.getMappedValueFromHandle(handle);
    if (!thermostat) {
        g_logger.logError('ThermostatConnectionOnCommRx: Error retrieving device from handle: ' + handle);
        return;
    }
    thermostat.OnCommRx(data);
}
function ThermostatOnPollingIntervalElapsed(handle) {
    g_logger.logTrace('ThermostatOnPollingIntervalElapsed');
    var thermostat = g_thermostatGlobalHandlerMap.getMappedValueFromHandle(handle);
    if (!thermostat) {
        g_logger.logError('ThermostatOnPollingIntervalElapsed: Error retrieving device from handle: ' + handle);
        return;
    }
    thermostat.OnPollingIntervalElapsed();
}
//#endregion
function cancelOverride(index) {
    g_logger.logTrace('cancelOverride: ' + index);
    g_thermostats[index].SetOverrideRemaining(0, 0);
}
function decrementSetpoint(index) {
    g_logger.logTrace('decrementSetpoint: ' + index);
    g_thermostats[index].DecrementCurrentSetpoint();
}
function incrementSetpoint(index) {
    g_logger.logTrace('incrementSetpoint: ' + index);
    g_thermostats[index].IncrementCurrentSetpoint();
}
function sendRawCommand(index, command) {
    g_logger.logTrace('incrementSetpoint: ' + index);
    g_thermostats[index].SendRawCommand(command);
}
function setControlMode(index, isManual) {
    g_logger.logTrace('setControlMode: ' + index);
    g_thermostats[index].SetControlMode(isManual ? 'ON' : 'OFF');
}
function SetFanMode(index, mode) {
    g_logger.logTrace('SetFanMode: ' + index);
    g_thermostats[index].SetFanMode(mode);
}
function setOccupancy(index, isOccupied) {
    g_logger.logTrace('setOccupied: ' + index);
    g_thermostats[index].SetOccupancy(isOccupied ? 'DAY' : 'NIGHT');
}
function setOperatingMode(index, mode) {
    g_logger.logTrace('SetOperatingMode: ' + index);
    g_thermostats[index].SetOperatingMode(mode);
}
function setOverrideFanMode(index, mode) {
    g_logger.logTrace('SetOverrideFanMode: ' + index);
    g_thermostats[index].OverrideFanMode(mode);
}
function setOverrideOperatingMode(index, mode) {
    g_logger.logTrace('SetOverrideOperatingMode: ' + index);
    g_thermostats[index].OverrideOperatingMode(mode);
}
function setOverrideRemaining(index, hours, minutes) {
    g_logger.logTrace('setOverrideRemaining: ' + index);
    g_thermostats[index].SetOverrideRemaining(hours, minutes);
}
function setSetpoint(index, mode, isOccupied, temperature) {
    g_logger.logTrace('setSetpoint: ' + index);
    var thermostat = g_thermostats[index];
    switch (mode) {
        case 'HEAT':
            if (isOccupied) {
                thermostat.SetOccupiedCoolSetpoint(temperature);
            }
            else {
                thermostat.SetOccupiedHeatSetpoint(temperature);
            }
            break;
        case 'COOL':
            if (isOccupied) {
                thermostat.SetUnoccupiedCoolSetpoint(temperature);
            }
            else {
                thermostat.SetUnoccupiedHeatSetpoint(temperature);
            }
            break;
        default:
            g_logger.logError('Unexpected value (' + mode + ')');
            break;
    }
}
